#include "X3D_DFSRenderingWalker.h"
#include "X3D_X3DComposedGeometryNode.h"
#include "X3D_IndexedLineSet.h"
#include "X3D_PointSet.h"

namespace X3DTK {
namespace X3D {

DFSRenderingWalker::DFSRenderingWalker()
: X3DComponentWalker()
{
  defineComponentName("Rendering", "X3D");

  defineWalkingFunction(&DFSRenderingWalker::walkX3DComposedGeometryNode);
  defineWalkingFunction(&DFSRenderingWalker::walkIndexedLineSet);
  defineWalkingFunction(&DFSRenderingWalker::walkPointSet);
}

DFSRenderingWalker::~DFSRenderingWalker()
{
}

void DFSRenderingWalker::walkX3DComposedGeometryNode(X3DComposedGeometryNode *N) const
{
  visitor->enter(N);
  
  SFNode C;
  C = N->getColor();
  if ((C != 0) && (visitor->walkOn(N, C)))
    walker->walk(C);
    
  C = N->getCoord();
  if ((C != 0) && (visitor->walkOn(N, C)))
    walker->walk(C);
    
  C = N->getNormal();
  if ((C != 0) && (visitor->walkOn(N, C)))
    walker->walk(C);
    
  C = N->getTexCoord();
  if ((C != 0) && (visitor->walkOn(N, C)))
    walker->walk(C);    
  
  visitor->leave(N);
}

void DFSRenderingWalker::walkIndexedLineSet(IndexedLineSet *I) const
{
  visitor->enter(I);
  
  SFNode C;
  C = I->getColor();
  if ((C != 0) && (visitor->walkOn(I, C)))
    walker->walk(C);
    
  C = I->getCoord();
  if ((C != 0) && (visitor->walkOn(I, C)))
    walker->walk(C);
    
  visitor->leave(I);
}

void DFSRenderingWalker::walkPointSet(PointSet *P) const
{
  visitor->enter(P);
  
  SFNode C;
  C = P->getColor();
  if ((C != 0) && (visitor->walkOn(P, C)))
    walker->walk(C);
    
  C = P->getCoord();
  if ((C != 0) && (visitor->walkOn(P, C)))
    walker->walk(C);
    
  visitor->leave(P);
}

}
}
