#include "X3D_MeshBuilderShapeVisitor.h"
#include "X3D_Appearance.h"
#include "X3D_IndexedFaceSet.h"
#include "X3D_FillProperties.h"
#include "X3D_LineProperties.h"
#include "X3D_Material.h"
#include "X3D_Shape.h"
#include "X3D_X3DGeometryNode.h"
#include "MESH_X3DGroupingNode.h"
#include "MESH_Shape.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

MeshBuilderShapeVisitor::MeshBuilderShapeVisitor()
: ShapeVisitor()
{
  defineEnterFunction(&MeshBuilderShapeVisitor::enterShape);
  defineEnterFunction(&MeshBuilderShapeVisitor::enterX3DAppearanceNode);
  defineWalkOnFunction(&MeshBuilderShapeVisitor::walkOnX3DAppearanceNode);
  defineLeaveFunction(&MeshBuilderShapeVisitor::leaveShape);
  defineLeaveFunction(&MeshBuilderShapeVisitor::leaveX3DAppearanceNode);

  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<MeshBuilderStateVariables>();
}

MeshBuilderShapeVisitor::~MeshBuilderShapeVisitor()
{
}

void MeshBuilderShapeVisitor::enterShape(Shape *S) const
{
  if (dynamic_cast<IndexedFaceSet *>(S->getGeometry()) != 0)
  {
    // If Shape has an IndexedFaceSet as a child, then we build the Mesh.
    MESH::SFNode MS = stateVariables->getNode(S);
    if (MS == 0)
    {
      MS = new MESH::Shape();
      MS->setName(S->getName());
      
      stateVariables->addCoupleNode(S, MS);
    }
    stateVariables->pushNode(MS);
  }
  else
  {
    // Shape has not an IndexedFaceSet as a child, but a Cylinder, Sphere...
    stateVariables->disableChildrenVisit();
    X3DNode *NS = static_cast<X3DNode *>(stateVariables->clone(S));
  
    MESH::X3DGroupingNode *G = dynamic_cast<MESH::X3DGroupingNode *>(stateVariables->getTop());
    if (G != 0)
      G->addChild(NS);
  }  
}

void MeshBuilderShapeVisitor::enterX3DAppearanceNode(X3DAppearanceNode *N) const
{
  X3DAppearanceNode *NN = static_cast<X3DAppearanceNode *>(stateVariables->clone(N));
  
  MESH::Shape *S = dynamic_cast<MESH::Shape *>(stateVariables->getTop());
  if (S != 0)
  {
    S->setAppearance(NN);
  }  
}

bool MeshBuilderShapeVisitor::walkOnX3DAppearanceNode(X3DAppearanceNode *, SFAbstractNode) const
{
  return false;
}

void MeshBuilderShapeVisitor::leaveShape(Shape *S) const
{
  if (dynamic_cast<IndexedFaceSet *>(S->getGeometry()) != 0)
    stateVariables->popNode();
  else
    stateVariables->enableChildrenVisit();  
}


void MeshBuilderShapeVisitor::leaveX3DAppearanceNode(X3DAppearanceNode *) const
{
}
  
}
}
